'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelPluginSyntaxFlow = require('babel-plugin-syntax-flow');

var _babelPluginSyntaxFlow2 = _interopRequireDefault(_babelPluginSyntaxFlow);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelAddFlowComments = require('babel-add-flow-comments');

var _babelAddFlowComments2 = _interopRequireDefault(_babelAddFlowComments);

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _uppercamelcase = require('uppercamelcase');

var _uppercamelcase2 = _interopRequireDefault(_uppercamelcase);

var _s2sUtils = require('s2s-utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const createUnion = union => (0, _s2sUtils.template)(`export type Action = UNION`)({
  UNION: t.unionTypeAnnotation(union)
});

const createInitAction = (0, _s2sUtils.template)(`export type ReduxInitAction = { type: '@@INIT' }`);

function createActionName(filename) {
  return `${(0, _uppercamelcase2.default)((0, _s2sUtils.getParentDirName)(filename))}Action`;
}

exports.default = () => {
  return {
    inherits: _babelPluginSyntaxFlow2.default,
    name: 's2s-action-root',
    visitor: {
      Program: {
        exit(programPath, { opts }) {
          const { input, output } = opts;
          const globOptions = Object.assign({ absolute: true }, opts.globOptions);

          if (!input) {
            throw new Error('require input option');
          }

          if (!output) {
            throw new Error('require output option');
          }

          const files = _globby2.default.sync(input, globOptions);

          const imports = files.map(f => (0, _s2sUtils.typeImport)(createActionName(f), 'Action', (0, _s2sUtils.getImportPath)(output, f)));

          const union = files.map(createActionName).map(name => t.genericTypeAnnotation(t.identifier(name)));

          const initAction = 'ReduxInitAction';
          union.unshift(t.genericTypeAnnotation(t.identifier(initAction)));

          const action = createUnion(union);

          programPath.node.body = [...imports, t.noop(), createInitAction(), t.noop(), action];

          (0, _babelAddFlowComments2.default)(programPath);
        }
      }
    }
  };
};